/***************************************************************************
 *            burning.c
 *
 *  Thu Jul 22 18:54:07 2004
 *  Copyright  2004  benjie
 ****************************************************************************/

#include <gtk/gtk.h>
#include <sys/types.h>
#include <unistd.h>
#include <ctype.h>
#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include "interface.h"
#include "support.h"
#include "tocdoc.h"
#include "timeline.h"
#include "audio.h"

#ifndef WIN32
#include <sys/wait.h>
#endif

GThread         *writeThread;
GMutex          *burnStatusMutex;
GtkWidget       *writingWindow;
gfloat           writeProgress;
gchar           *writeStatus;
gint		    progressTag;
gboolean        doneWriting;

extern prefsStruct prefs;

gint progressUpdate(gpointer data);
void *burnTocFile(void *filename);


void startBurn(gchar *filename) {
    writingWindow = GTK_WIDGET(create_writing());
    gtk_widget_show(writingWindow);
    doneWriting = FALSE;
    writeProgress = 0;
    writeStatus = NULL;
    burnStatusMutex = g_mutex_new();
    
    writeThread = g_thread_create(&burnTocFile, g_strdup(filename), TRUE, NULL);
    progressTag = g_timeout_add(250, progressUpdate, g_strdup(filename));
}


gint progressUpdate(gpointer data)
{
    gchar progressStr[7];
    
    if (doneWriting) {
	unlink(data);
        gtk_widget_hide(writingWindow);
        gtk_widget_destroy(writingWindow);
        g_mutex_free(burnStatusMutex);
        free(data);
		return FALSE;
	} else {
        g_mutex_lock(burnStatusMutex);
    	    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(lookup_widget(writingWindow, "progressbarWriting")),
	    	    						  writeProgress);
            sprintf(progressStr, "%#.1f%%", 100 * writeProgress);
    	    gtk_progress_bar_set_text(GTK_PROGRESS_BAR(lookup_widget(writingWindow, "progressbarWriting")),
	    	    						  progressStr);
            if (writeStatus) {
                gtk_label_set_text(GTK_LABEL(lookup_widget(writingWindow, "writeStatus")), writeStatus);
            }
        g_mutex_unlock(burnStatusMutex);
		return TRUE;
	}
}


gboolean
on_writing_delete_event                (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    doneWriting = TRUE;
    g_source_remove(progressTag);
    g_mutex_free(burnStatusMutex);
    return FALSE;
}


void *burnTocFile(void *filename){
    GPid   cdrdaoPid;
    gint   cdrdaoOut, cdrdaoErr;
    gint   cdrdaoRetVal;
    FILE   *fOut,     *fErr;
    gchar  *argv[10];
    gchar  outLine[256], errLine[256];
    gint   written, total;
    gint   i;

    // Set up cddao's arg list
    i = 0;
    argv[i++] = prefs.cdrdaoCmdStr;
    argv[i++] = "write";  // = "simulate"; for testing :)
    if (prefs.eject)
        argv[i++] = "--eject";
    argv[i++] = "--device";
    argv[i++] = prefs.deviceStr;
    argv[i++] = "--driver";
    argv[i++] = prefs.driverStr;
    argv[i++] = "-n";
    argv[i++] = (gchar*)filename;
    argv[i++] = NULL;
    
    /* Attempt to fork and check for errors */
    if (! g_spawn_async_with_pipes(NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD,
                NULL, NULL, &cdrdaoPid, NULL, &cdrdaoOut, &cdrdaoErr, NULL)) {
        fprintf(stderr,"Couldn't start cdrdao. Exiting.\n");  /* something went wrong */
        free(filename);
        doneWriting = TRUE;
        return 0;
    }

#ifndef WIN32
    fcntl(cdrdaoOut, F_SETFL, fcntl(cdrdaoOut, F_GETFL) | O_NONBLOCK);
    fcntl(cdrdaoErr, F_SETFL, fcntl(cdrdaoErr, F_GETFL) | O_NONBLOCK);
#endif
    fOut = fdopen(cdrdaoOut, "r");
    fErr = fdopen(cdrdaoErr, "r");
    
    outLine[0] = '\0';
    while(TRUE) {
#ifndef WIN32
        if (fgets(outLine, 255, fOut) == NULL) outLine[0] = '\0';
#endif
        if (fgets(errLine, 255, fErr) == NULL) errLine[0] = '\0';

        if (outLine[0] == '\0' && errLine[0] == '\0') {
            // Instead of usleep, use a select() above for better performance
            ms_sleep(100);
        } else {
            if (sscanf(errLine, "Wrote %d of %d MB", &written, &total) == 2) {
                g_mutex_lock(burnStatusMutex);
                    writeProgress = (gfloat)written/total;
                g_mutex_unlock(burnStatusMutex);
                outLine[0] = '\0';
#ifdef WIN32
                if (written == total) break;
#endif
            }
            if (outLine[0]!=0 && outLine[0]!='\r' && outLine[0]!='\n') {
                g_mutex_lock(burnStatusMutex);
                    if (writeStatus) free(writeStatus);
                    writeStatus = g_strndup(outLine, 255);
                g_mutex_unlock(burnStatusMutex);
            }
            if (errLine[0]!=0 && errLine[0]!='\r' && errLine[0]!='\n') {
                g_mutex_lock(burnStatusMutex);
                    if (writeStatus) free(writeStatus);
                    writeStatus = g_strndup(errLine, 255);
                g_mutex_unlock(burnStatusMutex);
            }
        }
#ifndef WIN32
        if (waitpid(cdrdaoPid, &cdrdaoRetVal, WNOHANG) != 0) break;
#endif
     }
#ifndef WIN32
    wait(&cdrdaoRetVal);        /* Wait for child process to end */
    //g_spawn_close_pid(cdrdaoPid);
    fclose(fOut);
    fclose(fErr);
    close(cdrdaoOut);
    close(cdrdaoErr);

    if (cdrdaoRetVal != 0) {
        printf("cdrdao exited with error %d\n", cdrdaoRetVal);
    }
#endif

    doneWriting = TRUE;

    free(filename);
    return 0;
}


gchar *cygpathize(gchar *filename)
{
    gchar  *cygFilename;
    
    if (tolower(filename[0]) >= 'a' && tolower(filename[0]) <= 'z' &&
        filename[1] == ':' && filename[2] == '\\') {
            cygFilename = g_strdup_printf("/cygdrive/%c/%s", filename[0],
                g_strdelimit(filename+3, "\\", '/'));
    } else {
        cygFilename = g_strdup(filename);
        cygFilename = g_strdelimit(cygFilename, "\\", '/');
    }
    
    return cygFilename;
}
